/*! edgestack v0.19.12 by Sebastian Software <s.werner@sebastian-software.de> */
import { applyMiddleware, combineReducers, compose, createStore } from 'redux';
import thunk from 'redux-thunk';
import React from 'react';
import { connect } from 'react-redux';
import { BrowserRouter, withRouter } from 'react-router-dom';
import PropTypes from 'prop-types';
import reactTreeWalker from 'react-tree-walker';
import { ApolloClient, ApolloProvider, createBatchingNetworkInterface, createNetworkInterface } from 'react-apollo';
import 'isomorphic-fetch';
import areIntlLocalesSupported from 'intl-locales-supported';
import { IntlProvider, addLocaleData, injectIntl } from 'react-intl';
import _objectWithoutProperties from 'babel-runtime/helpers/objectWithoutProperties';
import { render } from 'react-dom';

var SET_PATH = "router/SET_PATH";
var REPLACE_PATH = "router/REPLACE_PATH";
var RECOVER_PATH = "/pingback/";

var initialState = {};

/**
 * [routerReducer description]
 * @param {[type]} [state=initialState] [description]
 * @param {[type]} action [description]
 * @return {[type]} [description]
 */
function routerReducer() {
  var state = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : initialState,
      action = arguments[1];

  switch (action.type) {
    case SET_PATH:
      return Object.assign({}, state, { path: action.path, replace: false });
    case REPLACE_PATH:
      return Object.assign({}, state, { path: action.path, replace: true });
    default:
      return state;
  }
}

/**
 * [setPath description]
 * @param {[type]} path [description]
 */
function _setPath(path) {
  return {
    type: SET_PATH,
    path
  };
}

/**
 * [replacePath description]
 * @param {[type]} path [description]
 */


class RoutingConnector extends React.Component {
  componentDidMount() {
    if (this.context.router.history.location.pathname === this.props.recoverPath) {
      this.context.router.history.replace(this.props.path);
    } else {
      this.props.setPath(this.props.location.pathname);
    }
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.path === nextProps.location.pathname) {
      return;
    }

    // Sync from Redux
    if (nextProps.path !== this.props.path) {
      if (true === nextProps.replace) {
        this.context.router.history.replace(nextProps.path);
      } else {
        this.context.router.history.push(nextProps.path);
      }
    }

    // Sync to Redux
    if (nextProps.location !== this.props.location) {
      this.props.setPath(nextProps.location.pathname);
    }
  }

  render() {
    return this.props.children;
  }
}

RoutingConnector.defaultProps = {
  recoverPath: RECOVER_PATH
};

RoutingConnector.contextTypes = {
  router: PropTypes.object
};

function mapStateToProps(state) {
  return {
    path: state.router.path,
    replace: state.router.replace
  };
}

function mapDispatchToProps(dispatch) {
  return {
    setPath: path => dispatch(_setPath(path))
  };
}

var RouterConnector = withRouter(connect(mapStateToProps, mapDispatchToProps)(RoutingConnector));

var composeEnhancers = "development" === process.env.NODE_ENV && window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ || compose;

/**
 * Placeholder for a non active reducer in Redux.
 */
function emptyReducer() {
  var previousState = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      action = arguments[1];

  return previousState;
}

/**
 * Placeholder for a non active middleware in Redux.
 */
function emptyMiddleware() {
  return next => {
    return action => {
      return next(action);
    };
  };
}

/**
 * Placeholder for a non active enhancer in Redux.
 */
function emptyEnhancer(param) {
  return param;
}

/**
 * Dummy reducer for exporting server-side data to the client-side application.
 */
function ssrReducer() {
  var previousState = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      action = arguments[1];

  return previousState;
}

/**
 * Selector for quering the current locale e.g. de-DE, en-US, ...
 */
function getLocale(state) {
  return state.ssr.locale;
}

/**
 * Selector for quering the current language e.g. de, en, fr, es, ...
 */
function getLanguage(state) {
  return state.ssr.language;
}

/**
 * Selector for quering the current region e.g. DE, BR, PT, ...
 */
function getRegion(state) {
  return state.ssr.region;
}

/**
 * Bundles the given reducers into a root reducer for the application
 */
function createRootReducer(reducers) {
  return combineReducers(Object.assign({}, reducers, {
    ssr: ssrReducer,
    router: routerReducer
  }));
}

/**
 *
 *
 */
function createReduxStore() {
  var config = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      _config$reducers = config.reducers,
      reducers = _config$reducers === undefined ? {} : _config$reducers,
      _config$middlewares = config.middlewares,
      middlewares = _config$middlewares === undefined ? [] : _config$middlewares,
      _config$enhancers = config.enhancers,
      enhancers = _config$enhancers === undefined ? [] : _config$enhancers,
      initialState = config.initialState,
      apolloClient = config.apolloClient,
      rootReducer = apolloClient ? createRootReducer(Object.assign({}, reducers, { apollo: apolloClient.reducer() })) : createRootReducer(reducers),
      rootEnhancers = composeEnhancers(applyMiddleware(apolloClient ? apolloClient.middleware() : emptyMiddleware,

  // Redux middleware that spits an error on you when you try to mutate
  // your state either inside a dispatch or between dispatches.
  // https://github.com/leoasis/redux-immutable-state-invariant
  "development" === process.env.NODE_ENV ? require("redux-immutable-state-invariant")["default"]() : emptyMiddleware, thunk, ...middlewares,

  // Add automatic state change logging for client application
  // Note: Logger must be the last middleware in chain, otherwise it will log thunk and
  // promise, not actual actions (https://github.com/evgenyrodionov/redux-logger/issues/20).
  require("redux-logger").createLogger({ collapsed: true })), ...enhancers),
      store = createStore(rootReducer, initialState, rootEnhancers);


  return store;
}

/* eslint-disable no-shadow */
/* eslint-disable max-params */
function deepFetch(rootElement) {

  return reactTreeWalker(rootElement, function (element, instance) {
    if (instance && "function" == typeof instance.fetchData) {
      return instance.fetchData();
    }

    return true;
  });
}

/**
 *
 *
 */
function createApolloClient() {
  var config = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      headers = config.headers,
      _config$initialState = config.initialState,
      initialState = _config$initialState === undefined ? {} : _config$initialState,
      _config$batchRequests = config.batchRequests,
      batchRequests = _config$batchRequests === undefined ? false : _config$batchRequests,
      _config$trustNetwork = config.trustNetwork,
      trustNetwork = _config$trustNetwork === undefined ? true : _config$trustNetwork,
      _config$queryDeduplic = config.queryDeduplication,
      queryDeduplication = _config$queryDeduplic === undefined ? true : _config$queryDeduplic,
      apolloUri = initialState.ssr && initialState.ssr.apolloUri,
      ssrMode = "web" == "node",
      client = void 0;


  if (null != apolloUri) {
    var opts = {
      credentials: trustNetwork ? "include" : "same-origin",

      // transfer request headers to networkInterface so that they're accessible to proxy server
      // Addresses this issue: https://github.com/matthew-andrews/isomorphic-fetch/issues/83
      headers
    },
        networkInterface;

    if (batchRequests) {
      networkInterface = createBatchingNetworkInterface({
        uri: apolloUri,
        batchInterval: 10,
        opts
      });
    } else {
      networkInterface = createNetworkInterface({
        uri: apolloUri,
        opts
      });
    }

    client = new ApolloClient({
      ssrMode,
      queryDeduplication,
      networkInterface
    });
  } else {
    client = new ApolloClient({
      ssrMode,
      queryDeduplication
    });
  }

  return client;
}

var nonce = void 0;

nonce = document.querySelector("script[nonce]").getAttribute("nonce");


function injectCode(_ref) {
  var code = _ref.code,
      url = _ref.url;

  return new Promise((resolve, reject) => {
    var result = false,
        injectReference = document.getElementsByTagName("script")[0],
        scriptElem = document.createElement("script");


    if (url) {
      scriptElem.src = url;
    } else if (code) {
      scriptElem.innerText = code;
    }

    scriptElem.async = true;
    scriptElem.setAttribute("nonce", nonce);

    scriptElem.onload = scriptElem.onreadystatechange = () => {
      if (!result && (!this.readyState || "complete" === this.readyState)) {
        result = true;
        resolve(true);
      }
    };
    scriptElem.onerror = scriptElem.onabort = reject;
    injectReference.parentNode.insertBefore(scriptElem, injectReference);
  });
}

function ensureReactIntlSupport(language) {

  var reactIntlUrl = require("react-intl/locale-data/" + language + ".js");
  console.log("Loading React-Intl Data:", reactIntlUrl);
  return fetch(reactIntlUrl).then(response => {
    return response.text().then(code => {
      // This stuff is crappy but it's a way to work with the UMD packages of React-Intl.
      // See also: https://github.com/yahoo/react-intl/issues/853
      injectCode({ code });
      addLocaleData(ReactIntlLocaleData[language]);
    });
  });
}

function ensureIntlSupport(locale) {
  // Determine if the built-in `Intl` has the locale data we need.
  if (global.Intl && areIntlLocalesSupported([locale])) {
    return Promise.resolve(false);
  }

  // By default Node only ships with basic English locale data. You can however build a
  // Node binary with all locale data. We recommend doing this if you control the container
  // your Node app runs in, otherwise you'll want to polyfill Intl in Node.
  // Via: https://github.com/yahoo/react-intl/wiki#i18n-in-javascript


  var intlUrl = require("lean-intl/locale-data/" + locale + ".json");

  return new Promise(resolve => {
    require.ensure([], require => {
      resolve(require("lean-intl"));
    });
  }).then(IntlPolyfill => {
    return fetch(intlUrl).then(response => {
      return response.json().then(parsed => {
        IntlPolyfill.__addLocaleData(parsed

        // `Intl` exists, but it doesn't have the data we need, so load the
        // polyfill and patch the constructors we need with the polyfill's.
        );if (global.Intl) {
          Intl.NumberFormat = IntlPolyfill.NumberFormat;
          Intl.DateTimeFormat = IntlPolyfill.DateTimeFormat;
        } else {
          global.Intl = IntlPolyfill;
        }

        return Promise.resolve(true);
      });
    });
  });
}

function createLazyComponent(options) {
  var LazyLoading = false,
      LazyPromise = null,
      LazyComponent = null,
      LazyMessages = null;


  class LazyComponentWrapper extends React.Component {
    constructor(props) {
      super(props);

      this.state = {
        loading: LazyLoading
      };
    }

    componentDidMount() {
      // This callback is only executed on the client side
      // We have to manually call fetchData there as this is our own infrastructure for
      // loading data and typically not being executed on the client by the normal React render flow.
      if (!LazyPromise) {
        this.fetchData();
      }
    }

    fetchData() {
      if (false === this.state.loading && null == LazyComponent) {
        // Toggle loading state even when this is not the initating instance
        this.setState(() => ({
          loading: true
        }));
      }

      // Return existing promise from some other initiator... or when called two times in a row.
      if (null != LazyPromise) {
        return LazyPromise;
      }

      LazyLoading = true;
      LazyPromise = Promise.all(options.load(this.props.language)).then(result => {
        LazyLoading = false;

        // Map component. We have to use the default export, because import() does not
        // do this automatically. We also directly inject the intl object for easy
        // message translation via API.
        LazyComponent = injectIntl(result[0]["default"]

        // Store the messages of the given language for later access
        );LazyMessages = result[1];

        // Stop loading indicator
        this.setState(() => ({
          loading: LazyLoading
        }));

        return true;
      });

      return LazyPromise;
    }

    render() {
      if (!LazyComponent) {
        return null;
      }

      var _props = this.props,
          locale = _props.locale,
          props = _objectWithoutProperties(_props, ["locale"]),
          IntlWrappedComponent = React.createElement(
        IntlProvider,
        { locale: locale, messages: LazyMessages },
        React.createElement(LazyComponent, props)
      );

      return IntlWrappedComponent;
    }
  }

  return connect(state => ({
    locale: getLocale(state),
    language: getLanguage(state)
  }))(LazyComponentWrapper);
}

function renderApp(AppRoot, _ref) {
  var apolloClient = _ref.apolloClient,
      reduxStore = _ref.reduxStore,
      RoutedAppRoot = withRouter(AppRoot),
      WrappedRoot = React.createElement(
    ApolloProvider,
    { client: apolloClient, store: reduxStore },
    React.createElement(
      BrowserRouter,
      null,
      React.createElement(
        RouterConnector,
        null,
        React.createElement(RoutedAppRoot, null)
      )
    )
  );


  return deepFetch(WrappedRoot).then(() => render(WrappedRoot, document.getElementById("app")));
}

// This file is just for exporting infrastructure to applications built upon this.

export { renderApp, createReduxStore, createRootReducer, emptyReducer, emptyMiddleware, emptyEnhancer, ssrReducer, getRegion, getLanguage, getLocale, deepFetch, createApolloClient, RouterConnector, routerReducer, ensureIntlSupport, ensureReactIntlSupport, createLazyComponent };
//# sourceMappingURL=browser.modern.esmodule.js.map
