/*! edgestack v0.19.12 by Sebastian Software <s.werner@sebastian-software.de> */
import _toConsumableArray from 'babel-runtime/helpers/toConsumableArray';
import { applyMiddleware, combineReducers, compose, createStore } from 'redux';
import thunk from 'redux-thunk';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React from 'react';
import { connect } from 'react-redux';
import { withRouter } from 'react-router-dom';
import PropTypes from 'prop-types';
import reactTreeWalker from 'react-tree-walker';
import { ApolloClient, ApolloProvider, createBatchingNetworkInterface, createNetworkInterface } from 'react-apollo';
import 'isomorphic-fetch';
import areIntlLocalesSupported from 'intl-locales-supported';
import { IntlProvider, injectIntl } from 'react-intl';
import _objectWithoutProperties from 'babel-runtime/helpers/objectWithoutProperties';
import express from 'express';
import shrinkRay from 'shrink-ray';
import uuid from 'uuid';
import parameterProtection from 'hpp';
import helmet from 'helmet';
import PrettyError from 'pretty-error';
import favicon from 'serve-favicon';
import createLocaleMiddleware from 'express-locale';
import cookieParser from 'cookie-parser';
import bodyParser from 'body-parser';
import { resolve } from 'path';
import appRootDir from 'app-root-dir';
import { renderToString } from 'react-dom/server';
import { StaticRouter } from 'react-router';
import Helmet from 'react-helmet';
import serialize from 'serialize-javascript';
import { readFileSync } from 'fs';
import codeFrame from 'babel-code-frame';
import chalk from 'chalk';
import { wrapCallSite } from 'source-map-support';

var SET_PATH = "router/SET_PATH";
var REPLACE_PATH = "router/REPLACE_PATH";
var RECOVER_PATH = "/pingback/";

var initialState = {};

/**
 * [routerReducer description]
 * @param {[type]} [state=initialState] [description]
 * @param {[type]} action [description]
 * @return {[type]} [description]
 */
function routerReducer() {
  var state = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : initialState,
      action = arguments[1];

  switch (action.type) {
    case SET_PATH:
      return Object.assign({}, state, { path: action.path, replace: false });
    case REPLACE_PATH:
      return Object.assign({}, state, { path: action.path, replace: true });
    default:
      return state;
  }
}

/**
 * [setPath description]
 * @param {[type]} path [description]
 */
function _setPath(path$$1) {
  return {
    type: SET_PATH,
    path: path$$1
  };
}

/**
 * [replacePath description]
 * @param {[type]} path [description]
 */


var RoutingConnector = function (_React$Component) {
  _inherits(RoutingConnector, _React$Component);

  function RoutingConnector() {
    _classCallCheck(this, RoutingConnector);

    return _possibleConstructorReturn(this, (RoutingConnector.__proto__ || Object.getPrototypeOf(RoutingConnector)).apply(this, arguments));
  }

  _createClass(RoutingConnector, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      if (this.context.router.history.location.pathname === this.props.recoverPath) {
        this.context.router.history.replace(this.props.path);
      } else {
        this.props.setPath(this.props.location.pathname);
      }
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.path === nextProps.location.pathname) {
        return;
      }

      // Sync from Redux
      if (nextProps.path !== this.props.path) {
        if (true === nextProps.replace) {
          this.context.router.history.replace(nextProps.path);
        } else {
          this.context.router.history.push(nextProps.path);
        }
      }

      // Sync to Redux
      if (nextProps.location !== this.props.location) {
        this.props.setPath(nextProps.location.pathname);
      }
    }
  }, {
    key: "render",
    value: function render() {
      return this.props.children;
    }
  }]);

  return RoutingConnector;
}(React.Component);

RoutingConnector.defaultProps = {
  recoverPath: RECOVER_PATH
};

RoutingConnector.contextTypes = {
  router: PropTypes.object
};

function mapStateToProps(state) {
  return {
    path: state.router.path,
    replace: state.router.replace
  };
}

function mapDispatchToProps(dispatch) {
  return {
    setPath: function setPath(path$$1) {
      return dispatch(_setPath(path$$1));
    }
  };
}

var RouterConnector = withRouter(connect(mapStateToProps, mapDispatchToProps)(RoutingConnector));

var composeEnhancers = "web" == "node" || compose;

/**
 * Placeholder for a non active reducer in Redux.
 */
function emptyReducer() {
  var previousState = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      action = arguments[1];

  return previousState;
}

/**
 * Placeholder for a non active middleware in Redux.
 */
function emptyMiddleware() {
  return function (next) {
    return function (action) {
      return next(action);
    };
  };
}

/**
 * Placeholder for a non active enhancer in Redux.
 */
function emptyEnhancer(param) {
  return param;
}

/**
 * Dummy reducer for exporting server-side data to the client-side application.
 */
function ssrReducer() {
  var previousState = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      action = arguments[1];

  return previousState;
}

/**
 * Selector for quering the current locale e.g. de-DE, en-US, ...
 */
function getLocale(state) {
  return state.ssr.locale;
}

/**
 * Selector for quering the current language e.g. de, en, fr, es, ...
 */
function getLanguage(state) {
  return state.ssr.language;
}

/**
 * Selector for quering the current region e.g. DE, BR, PT, ...
 */
function getRegion(state) {
  return state.ssr.region;
}

/**
 * Bundles the given reducers into a root reducer for the application
 */
function createRootReducer(reducers) {
  return combineReducers(Object.assign({}, reducers, {
    ssr: ssrReducer,
    router: routerReducer
  }));
}

/**
 *
 *
 */
function createReduxStore() {
  var config = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      _config$reducers = config.reducers,
      reducers = _config$reducers === undefined ? {} : _config$reducers,
      _config$middlewares = config.middlewares,
      middlewares = _config$middlewares === undefined ? [] : _config$middlewares,
      _config$enhancers = config.enhancers,
      enhancers = _config$enhancers === undefined ? [] : _config$enhancers,
      initialState = config.initialState,
      apolloClient = config.apolloClient,
      rootReducer = apolloClient ? createRootReducer(Object.assign({}, reducers, { apollo: apolloClient.reducer() })) : createRootReducer(reducers),
      rootEnhancers = composeEnhancers.apply(undefined, [applyMiddleware.apply(undefined, [apolloClient ? apolloClient.middleware() : emptyMiddleware,

  // Redux middleware that spits an error on you when you try to mutate
  // your state either inside a dispatch or between dispatches.
  // https://github.com/leoasis/redux-immutable-state-invariant
  "development" === process.env.NODE_ENV ? require("redux-immutable-state-invariant")["default"]() : emptyMiddleware, thunk].concat(_toConsumableArray(middlewares), [

  // Add automatic state change logging for client application
  // Note: Logger must be the last middleware in chain, otherwise it will log thunk and
  // promise, not actual actions (https://github.com/evgenyrodionov/redux-logger/issues/20).
  emptyMiddleware]))].concat(_toConsumableArray(enhancers))),
      store = createStore(rootReducer, initialState, rootEnhancers);


  return store;
}

/* eslint-disable no-shadow */
/* eslint-disable max-params */
function deepFetch(rootElement) {

  return reactTreeWalker(rootElement, function (element, instance) {
    if (instance && "function" == typeof instance.fetchData) {
      return instance.fetchData();
    }

    return true;
  });
}

/**
 *
 *
 */
function createApolloClient() {
  var config = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : {},
      headers = config.headers,
      _config$initialState = config.initialState,
      initialState = _config$initialState === undefined ? {} : _config$initialState,
      _config$batchRequests = config.batchRequests,
      batchRequests = _config$batchRequests === undefined ? false : _config$batchRequests,
      _config$trustNetwork = config.trustNetwork,
      trustNetwork = _config$trustNetwork === undefined ? true : _config$trustNetwork,
      _config$queryDeduplic = config.queryDeduplication,
      queryDeduplication = _config$queryDeduplic === undefined ? true : _config$queryDeduplic,
      apolloUri = initialState.ssr && initialState.ssr.apolloUri,
      ssrMode = "node" == "node",
      client = void 0;


  if (null != apolloUri) {
    var opts = {
      credentials: trustNetwork ? "include" : "same-origin",

      // transfer request headers to networkInterface so that they're accessible to proxy server
      // Addresses this issue: https://github.com/matthew-andrews/isomorphic-fetch/issues/83
      headers: headers
    },
        networkInterface;

    if (batchRequests) {
      networkInterface = createBatchingNetworkInterface({
        uri: apolloUri,
        batchInterval: 10,
        opts: opts
      });
    } else {
      networkInterface = createNetworkInterface({
        uri: apolloUri,
        opts: opts
      });
    }

    client = new ApolloClient({
      ssrMode: ssrMode,
      queryDeduplication: queryDeduplication,
      networkInterface: networkInterface
    });
  } else {
    client = new ApolloClient({
      ssrMode: ssrMode,
      queryDeduplication: queryDeduplication
    });
  }

  return client;
}

function ensureReactIntlSupport(language) {
  // Locale Data in Node.js:
  // When using React Intl in Node.js (same for the Intl.js polyfill), all locale data will be
  // loaded into memory. This makes it easier to write a universal/isomorphic React app with
  // React Intl since you won't have to worry about dynamically loading locale data on the server.
  // Via: https://github.com/yahoo/react-intl/wiki#locale-data-in-nodejs

  // As mentioned above no additional data has to be loaded for NodeJS. We are just resolving
  // the Promise in that case.
  return Promise.resolve(false);


  var reactIntlUrl = require("react-intl/locale-data/" + language + ".js");
}

function ensureIntlSupport(locale) {
  // Determine if the built-in `Intl` has the locale data we need.
  if (global.Intl && areIntlLocalesSupported([locale])) {
    return Promise.resolve(false);
  }

  // By default Node only ships with basic English locale data. You can however build a
  // Node binary with all locale data. We recommend doing this if you control the container
  // your Node app runs in, otherwise you'll want to polyfill Intl in Node.
  // Via: https://github.com/yahoo/react-intl/wiki#i18n-in-javascript

  /* eslint-disable no-console */
  console.warn("Your NodeJS installation does not include full ICU locale data! Fallback to polyfill!");
  console.warn("See also: https://github.com/nodejs/node/wiki/Intl");


  var intlUrl = require("lean-intl/locale-data/" + locale + ".json");

  return new Promise(function (resolve$$1) {
    require.ensure([], function (require) {
      resolve$$1(require("lean-intl"));
    });
  }).then(function (IntlPolyfill) {
    return fetch(intlUrl).then(function (response) {
      return response.json().then(function (parsed) {
        IntlPolyfill.__addLocaleData(parsed

        // `Intl` exists, but it doesn't have the data we need, so load the
        // polyfill and patch the constructors we need with the polyfill's.
        );if (global.Intl) {
          Intl.NumberFormat = IntlPolyfill.NumberFormat;
          Intl.DateTimeFormat = IntlPolyfill.DateTimeFormat;
        } else {
          global.Intl = IntlPolyfill;
        }

        return Promise.resolve(true);
      });
    });
  });
}

function createLazyComponent(options) {
  var LazyLoading = false,
      LazyPromise = null,
      LazyComponent = null,
      LazyMessages = null;

  var LazyComponentWrapper = function (_React$Component) {
    _inherits(LazyComponentWrapper, _React$Component);

    function LazyComponentWrapper(props) {
      _classCallCheck(this, LazyComponentWrapper);

      var _this = _possibleConstructorReturn(this, (LazyComponentWrapper.__proto__ || Object.getPrototypeOf(LazyComponentWrapper)).call(this, props));

      _this.state = {
        loading: LazyLoading
      };
      return _this;
    }

    _createClass(LazyComponentWrapper, [{
      key: "componentDidMount",
      value: function componentDidMount() {
        // This callback is only executed on the client side
        // We have to manually call fetchData there as this is our own infrastructure for
        // loading data and typically not being executed on the client by the normal React render flow.
        if (!LazyPromise) {
          this.fetchData();
        }
      }
    }, {
      key: "fetchData",
      value: function fetchData() {
        var _this2 = this;

        if (false === this.state.loading && null == LazyComponent) {
          // Toggle loading state even when this is not the initating instance
          this.setState(function () {
            return {
              loading: true
            };
          });
        }

        // Return existing promise from some other initiator... or when called two times in a row.
        if (null != LazyPromise) {
          return LazyPromise;
        }

        LazyLoading = true;
        LazyPromise = Promise.all(options.load(this.props.language)).then(function (result) {
          LazyLoading = false;

          // Map component. We have to use the default export, because import() does not
          // do this automatically. We also directly inject the intl object for easy
          // message translation via API.
          LazyComponent = injectIntl(result[0]["default"]

          // Store the messages of the given language for later access
          );LazyMessages = result[1];

          // Stop loading indicator
          _this2.setState(function () {
            return {
              loading: LazyLoading
            };
          });

          return true;
        });

        return LazyPromise;
      }
    }, {
      key: "render",
      value: function render() {
        if (!LazyComponent) {
          return null;
        }

        var _props = this.props,
            locale = _props.locale,
            props = _objectWithoutProperties(_props, ["locale"]),
            IntlWrappedComponent = React.createElement(
          IntlProvider,
          { locale: locale, messages: LazyMessages },
          React.createElement(LazyComponent, props)
        );

        return IntlWrappedComponent;
      }
    }]);

    return LazyComponentWrapper;
  }(React.Component);

  return connect(function mapStateToProps(state) {
    return {
      locale: getLocale(state),
      language: getLanguage(state)
    };
  })(LazyComponentWrapper);
}

// Contains all the configuration specific to our Server.
// Please note that the DefinePlugin of webpack will inline/replace all the
// respective "process.env.*" variables below with the actual values.
// These config values then become a part of the server bundle.

var APP_ROOT = appRootDir.get();

var ABSOLUTE_CLIENT_OUTPUT_PATH = resolve(APP_ROOT, process.env.CLIENT_BUNDLE_OUTPUT_PATH);

var ABSOLUTE_ASSETSINFO_PATH = resolve(APP_ROOT, process.env.CLIENT_BUNDLE_OUTPUT_PATH, process.env.CLIENT_BUNDLE_ASSETS_FILENAME);

var ABSOLUTE_CHUNKMANIFEST_PATH = resolve(APP_ROOT, process.env.CLIENT_BUNDLE_OUTPUT_PATH, process.env.CLIENT_BUNDLE_CHUNK_MANIFEST_FILENAME);

var ABSOLUTE_PUBLIC_PATH = resolve(APP_ROOT, process.env.CLIENT_PUBLIC_PATH);

var pretty = new PrettyError();

// this will skip events.js and http.js and similar core node files
pretty.skipNodeFiles

// this will skip all the trace lines about express` core and sub-modules
();pretty.skipPackage("express");

function createExpressServer(config) {
  // Create our express based server.
  var server = express

  // Create new middleware to serve a favicon from the given path to a favicon file.
  ();server.use(favicon(ABSOLUTE_PUBLIC_PATH + "/favicon.ico")

  // Attach a unique "nonce" to every response. This allows use to declare
  // inline scripts as being safe for execution against our content security policy.
  // @see https://helmetjs.github.io/docs/csp/
  );server.use(function (request, response, next) {
    response.locals.nonce = uuid // eslint-disable-line no-param-reassign
    ();next();
  }

  // and use it for our app's error handler:
  );server.use(function (error, request, response, next) {
    // eslint-disable-line max-params
    console.log(pretty.render(error));
    next();
  }

  // Don't expose any software information to hackers.
  );server.disable("x-powered-by"

  // Prevent HTTP Parameter pollution.
  );server.use(parameterProtection()

  // Content Security Policy (CSP)
  //
  // If you are unfamiliar with CSPs then I highly recommend that you do some
  // reading on the subject:
  //  - https://content-security-policy.com/
  //  - https://developers.google.com/web/fundamentals/security/csp/
  //  - https://developer.mozilla.org/en/docs/Web/Security/CSP
  //  - https://helmetjs.github.io/docs/csp/
  //
  // If you are relying on scripts/styles/assets from other servers (internal or
  // external to your company) then you will need to explicitly configure the
  // CSP below to allow for this.  For example you can see I have had to add
  // the polyfill.io CDN in order to allow us to use the polyfill script.
  // It can be a pain to manage these, but it's a really great habit to get in
  // to.
  //
  // You may find CSPs annoying at first, but it is a great habit to build.
  // The CSP configuration is an optional item for helmet, however you should
  // not remove it without making a serious consideration that you do not require
  // the added security.
  );var cspConfig = {
    directives: {
      defaultSrc: ["'self'"],

      scriptSrc: [
      // Allow scripts hosted from our application.
      "'self'",

      // Note: We will execution of any inline scripts that have the following
      // nonce identifier attached to them.
      // This is useful for guarding your application whilst allowing an inline
      // script to do data store rehydration (redux/mobx/apollo) for example.
      // @see https://helmetjs.github.io/docs/csp/
      function (request, response) {
        return "'nonce-" + response.locals.nonce + "'";
      },

      // FIXME: Required for eval-source-maps (devtool in webpack)
      "development" === process.env.NODE_ENV ? "'unsafe-eval'" : ""].filter(function (value) {
        return "" !== value;
      }),

      styleSrc: ["'self'", "'unsafe-inline'", "blob:"],
      imgSrc: ["'self'", "data:"],
      fontSrc: ["'self'", "data:"],

      // Note: Setting this to stricter than * breaks the service worker. :(
      // I can't figure out how to get around this, so if you know of a safer
      // implementation that is kinder to service workers please let me know.
      // ["'self'", 'ws:'],
      connectSrc: ["*"],

      // objectSrc: [ "'none'" ],
      // mediaSrc: [ "'none'" ],

      childSrc: ["'self'"]
    }
  };

  if ("development" === process.env.NODE_ENV) {
    // When in development mode we need to add our secondary express server that
    // is used to host our client bundle to our csp config.
    Object.keys(cspConfig.directives).forEach(function (directive) {
      return cspConfig.directives[directive].push("localhost:" + process.env.CLIENT_DEVSERVER_PORT);
    });
  }

  server.use(helmet.contentSecurityPolicy(cspConfig)

  // The xssFilter middleware sets the X-XSS-Protection header to prevent
  // reflected XSS attacks.
  // @see https://helmetjs.github.io/docs/xss-filter/
  );server.use(helmet.xssFilter()

  // Frameguard mitigates clickjacking attacks by setting the X-Frame-Options header.
  // @see https://helmetjs.github.io/docs/frameguard/
  );server.use(helmet.frameguard("deny")

  // Sets the X-Download-Options to prevent Internet Explorer from executing
  // downloads in your site’s context.
  // @see https://helmetjs.github.io/docs/ienoopen/
  );server.use(helmet.ieNoOpen()

  // Don’t Sniff Mimetype middleware, noSniff, helps prevent browsers from trying
  // to guess (“sniff”) the MIME type, which can have security implications. It
  // does this by setting the X-Content-Type-Options header to nosniff.
  // @see https://helmetjs.github.io/docs/dont-sniff-mimetype/
  );server.use(helmet.noSniff());

  if (config.customMiddleware) config.customMiddleware.forEach(function (middleware) {
    if (middleware instanceof Array) server.use.apply(server, _toConsumableArray(middleware));else server.use(middleware);
  }

  // Parse cookies via standard express tooling
  );server.use(cookieParser()

  // Detect client locale and match it with configuration
  );server.use(createLocaleMiddleware({
    priority: ["query", "cookie", "accept-language", "default"],
    "default": config.DEFAULT_LOCALE.replace(/-/, "_"),
    allowed: config.ALLOWED_LOCALES.map(function (entry) {
      return entry.replace(/-/, "_");
    })
  })

  // Parse application/x-www-form-urlencoded
  );server.use(bodyParser.urlencoded({ extended: false })

  // Parse application/json
  );server.use(bodyParser.json()

  // Advanced response compression using a async zopfli/brotli combination
  // https://github.com/aickin/shrink-ray
  );server.use(shrinkRay()

  // Configure static serving of our webpack bundled client files.
  );server.use(process.env.CLIENT_BUNDLE_HTTP_PATH, express["static"](ABSOLUTE_CLIENT_OUTPUT_PATH, {
    maxAge: process.env.CLIENT_BUNDLE_CACHE_MAXAGE
  })

  // Configure static serving of our "public" root http path static files.
  );server.use(express["static"](ABSOLUTE_PUBLIC_PATH));

  return server;
}

var chunkManifest = "{}";
if ("production" === process.env.MODE) {
  try {
    chunkManifest = readFileSync(ABSOLUTE_CHUNKMANIFEST_PATH, "utf-8");
  } catch (error) {
    console.warn("Could not parse chunkhashes from manifest.json: ", error);
  }
}

var clientAssets = JSON.parse(readFileSync(ABSOLUTE_ASSETSINFO_PATH, "utf-8")

// Convert the assets json it into an object that contains all the paths to our
// javascript and css files. Doing this is required as for production
// configurations we add a hash to our filenames, therefore we won't know the
// paths of the output by webpack unless we read them from the assets.json file.
// const chunks = Object.keys(clientAssets).map((key) => clientAssets[key])

);function getAssetsForClientChunks(chunks) {
  return chunks.reduce(function (result, chunkName) {
    var chunkAssets = clientAssets[chunkName];
    if (chunkAssets) {
      if (chunkAssets.js) {
        result.scripts.push(chunkAssets.js);
      }
      if (chunkAssets.css) {
        result.styles.push(chunkAssets.css);
      }
    }
    return result;
  }, { scripts: [], styles: [] });
}

/**
 * Generates HTML link stylesheet tags from a list of stylesheet URLs.
 */
function generateLinkStyleTags(styles) {
  return styles.map(function (uri) {
    return "<link href=\"" + uri + "\" media=\"screen, projection\" rel=\"stylesheet\" />";
  }).join("\n");
}

/**
 * Generates HTML script tags from a list of script URLs.
 */
function generateScriptTags(scripts) {
  return scripts.map(function (uri) {
    return "<script src=\"" + uri + "\"></script>";
  }).join("\n");
}

/**
 * Generates a full HTML page containing the render output of the given react
 * element.
 *
 * @param  rootReactElement
 *   [Optional] The root React element to be rendered on the page.
 * @param  initialState
 *   [Optional] The initial state for the redux store which will be used by the
 *   client to mount the redux store into the desired state.
 *
 * @return The full HTML page in the form of a React element.
 */
function renderPage(_ref) {
  var renderedApp = _ref.renderedApp,
      _ref$initialState = _ref.initialState,
      initialState = _ref$initialState === undefined ? {} : _ref$initialState,
      nonce = _ref.nonce,
      helmet$$1 = _ref.helmet,
      codeSplitState = _ref.codeSplitState,
      STATE_IDENTIFIER = _ref.STATE_IDENTIFIER,
      language = _ref.language,
      region = _ref.region,
      assetsForRender = getAssetsForClientChunks([
  // We always manually add the main entry chunk for our client bundle. It
  // must always be the first item in the collection.
  "vendor", "main"]

  /* eslint-disable prefer-template */

  ),
      inlineCode = "APP_STATE=" + serialize(initialState, { isJSON: true }) + ";" + ("CHUNK_MANIFEST=" + chunkManifest + ";") + (null != STATE_IDENTIFIER ? STATE_IDENTIFIER + "=" + serialize(codeSplitState, { isJSON: true }) + ";" : ""),
      langValue = region ? language + "-" + region : language;
  // The chunks that we need to fetch the assets (js/css) for and then include
  // said assets as script/style tags within our html.


  /*
  if (codeSplitState && codeSplitState.chunks) {
    // We add all the chunks that our CodeSplitProvider tracked as being used
    // for this render. This isn't actually required as the rehydrate function
    // of code-split-component which gets executed in our client bundle will
    // ensure all our required chunks are loaded, but its a nice optimisation as
    // it means the browser can start fetching the required files before it's
    // even finished parsing our client bundle entry script.
    // Having the assets.json file available to us made implementing this
    // feature rather arbitrary.
    codeSplitState.chunks.forEach((chunk) => chunksForRender.push(chunk))
  }
  */

  // Now we get the assets (js/css) for the chunks.

  return "<!doctype html>\n    <html lang=\"" + langValue + "\" " + (helmet$$1 ? helmet$$1.htmlAttributes.toString() : "") + ">\n      <head>\n        " + (helmet$$1 ? helmet$$1.title.toString() : "") + "\n        " + (helmet$$1 ? helmet$$1.meta.toString() : "") + "\n        " + (helmet$$1 ? helmet$$1.link.toString() : "") + "\n\n        " + generateLinkStyleTags(assetsForRender.styles) + "\n        " + (helmet$$1 ? helmet$$1.style.toString() : "") + "\n      </head>\n      <body>\n        <div id=\"app\">" + (renderedApp || "") + "</div>\n\n        <script nonce=\"" + nonce + "\">" + inlineCode + "</script>\n\n        " + generateScriptTags(assetsForRender.scripts) + "\n        " + (helmet$$1 ? helmet$$1.script.toString() : "") + "\n      </body>\n    </html>";
}

/* eslint-disable no-console */
/**
 * Using Apollo logic to recursively resolve all queries needed for
 * initial rendering. The convention is to use the full JSX tree,
 * traverse it and look for all occurrences of a static `fetchData() => Promise`
 * method which is being executed and waited for.
 *
 * See also:
 * https://www.npmjs.com/package/react-redux-universal-hot-example#server-side-data-fetching
 */
function renderToStringWithData(component) {
  return deepFetch(component).then(function () {
    var result = renderToString(component);
    return result;
  });
}

// SSR is disabled so we will just return an empty html page and will
// rely on the client to populate the initial react application state.
function renderLight(_ref) {
  var request = _ref.request,
      response = _ref.response,
      nonce = _ref.nonce,
      initialState = _ref.initialState,
      language = _ref.language,
      region = _ref.region;

  /* eslint-disable no-magic-numbers */
  try {
    var html = renderPage({
      // Provide the redux store state, this will be bound to the window.APP_STATE
      // so that we can rehydrate the state on the client.
      initialState: initialState,

      // Nonce which allows us to safely declare inline scripts.
      nonce: nonce,

      // Send detected language and region for HTML tag info
      language: language,
      region: region
    });

    response.status(200).send(html);
  } catch (error) {
    response.status(500).send("Error during rendering: " + error + "!: " + error.stack);
  }
}

function renderFull(_ref2) {
  var request = _ref2.request,
      response = _ref2.response,
      nonce = _ref2.nonce,
      Root = _ref2.Root,
      apolloClient = _ref2.apolloClient,
      reduxStore = _ref2.reduxStore,
      language = _ref2.language,
      region = _ref2.region,
      routingContext = {};


  console.log("Server: Rendering app with data...");

  var WrappedRoot = React.createElement(
    StaticRouter,
    { location: request.url, context: routingContext },
    React.createElement(
      ApolloProvider,
      { client: apolloClient, store: reduxStore },
      React.createElement(Root, null)
    )
  );

  return Promise.all([ensureIntlSupport(language + "-" + region), ensureReactIntlSupport(language)]).then(function () {
    return renderToStringWithData(WrappedRoot);
  }

  // Create the application react element.
  ).then(function (renderedApp) {
    var reduxState = reduxStore.getState

    // Render the app to a string.
    (),
        html = renderPage({
      // Provide the full rendered App react element.
      renderedApp: renderedApp,

      // Provide the redux store state, this will be bound to the window.APP_STATE
      // so that we can rehydrate the state on the client.
      initialState: reduxState,

      // Nonce which allows us to safely declare inline scripts.
      nonce: nonce,

      // Running this gets all the helmet properties (e.g. headers/scripts/title etc)
      // that need to be included within our html. It's based on the rendered app.
      // @see https://github.com/nfl/react-helmet
      helmet: Helmet.rewind(),

      // Send detected language and region for HTML tag info
      language: language,
      region: region
    }

    // console.log("Server: Routing Context:", routingContext)
    );console.log("Server: Sending Page..."

    /* eslint-disable no-magic-numbers */

    // Check if the render result contains a redirect, if so we need to set
    // the specific status and redirect header and end the response.
    );if (routingContext.url) {
      response.status(302).setHeader("Location", routingContext.url);
      response.end();
      return;
    }

    // If the renderedResult contains a "missed" match then we set a 404 code.
    // Our App component will handle the rendering of an Error404 view.
    // Otherwise everything is all good and we send a 200 OK status.
    response.status(routingContext.status || 200).send(html);
  });
}

/**
 * An express middleware that is capable of doing React server side rendering.
 */
function createUniversalMiddleware$$1(_ref3) {
  var Root = _ref3.Root,
      State = _ref3.State,
      ssrData = _ref3.ssrData,
      _ref3$batchRequests = _ref3.batchRequests,
      batchRequests = _ref3$batchRequests === undefined ? false : _ref3$batchRequests,
      _ref3$trustNetwork = _ref3.trustNetwork,
      trustNetwork = _ref3$trustNetwork === undefined ? true : _ref3$trustNetwork;

  if (null == Root) {
    throw new Error("Server: Universal Middleware: Missing Root!");
  }

  if (null == State) {
    throw new Error("Server: Universal Middleware: Missing State!");
  }

  return function (request, response) {
    if ("string" != typeof response.locals.nonce) {
      throw new TypeError("Server: A \"nonce\" value has not been attached to the response");
    }
    var nonce = response.locals.nonce;

    if (!request.locale) {
      throw new Error("Cannot correctly deal with locale configuration!");
    }

    var _request$locale = request.locale,
        language = _request$locale.language,
        region = _request$locale.region;


    console.log("Incoming URL:", request.originalUrl, process.env.DISABLE_SSR ? "[SSR: disabled]" : "[SSR: enabled]", "[Locale: " + language + "-" + region + "]"

    // After matching locales with configuration we send the accepted locale
    // back to the client using a simple session cookie
    );response.cookie("locale", language + "-" + region

    // Pass object with all Server Side Rendering (SSR) related data to the client
    );var initialState = {
      ssr: Object.assign({}, ssrData, {
        locale: language + "-" + region,
        language: language,
        region: region
      })
    };

    if (process.env.DISABLE_SSR) {
      renderLight({
        request: request,
        response: response,
        nonce: nonce,
        initialState: initialState,
        language: language,
        region: region
      });
    } else {
      var apolloClient = createApolloClient({
        headers: request.headers,
        initialState: initialState,
        batchRequests: batchRequests,
        trustNetwork: trustNetwork
      }),
          reduxStore = createReduxStore({
        reducers: State.getReducers(),
        enhancers: State.getEnhancers(),
        middlewares: State.getMiddlewares(),
        apolloClient: apolloClient,
        initialState: initialState
      });

      renderFull({
        request: request,
        response: response,
        nonce: nonce,
        Root: Root,
        apolloClient: apolloClient,
        reduxStore: reduxStore,
        language: language,
        region: region
      });
    }
  };
}

/* eslint-disable no-magic-numbers, max-params */
function addFallbackHandler(server) {
  // Handle 404 errors.
  // Note: the react application middleware hands 404 paths, but it is good to
  // have this backup for paths not handled by the universal middleware. For
  // example you may bind a /api path to express.
  server.use(function (request, response) {
    // eslint-disable-line no-unused-vars,max-len
    response.status(404).send("Sorry, that resource was not found.");
  }

  // Handle all other errors (i.e. 500).
  // Note: You must provide specify all 4 parameters on this callback function
  // even if they aren't used, otherwise it won't be used.
  );server.use(function (error, request, response) {
    if (error) {
      console.log(error);
      console.log(error.stack);
    }

    response.status(500).send("Sorry, an unexpected error occurred.");
  });
}

var FUNCTION_NAME_FILTERS = [/__webpack_require__/];
var SOURCE_FILE_FILTERS = [/\/webpack\/bootstrap/];
var FUNCTION_NAME_CLEARANCE = [/__webpack_exports__/];
var TYPE_NAME_CLEARANCE = [/^Object$/];
var CODE_FRAME_OPTIONS = {
  highlightCode: true

  /**
   * Cleans up the source file name which was modified by `source-map-support` to point to the source mapped
   * origin file. Unfortunately instead of giving us the pure source file it resolves into something
   * like "<generatedFile>:<sourceFile>". We take care of this here and remove the generated file segment
   * as this is typically not relevant for the developer.
   * @param {String} sourceFile Sourcefile as being available in the CallSite
   * @return {String} Cleaned up plain source file reference
   */
};
function cleanSourceFileName(sourceFile) {
  var cleanFile = sourceFile.split(":")[1] || sourceFile;
  if ("/" === cleanFile.charAt(0)) {
    cleanFile = cleanFile.slice(1);
  }

  return cleanFile;
}

/**
 * This method checks whether the given stack frame (CallSite) is relevant from a
 * developer perspective.
 * @param {CallSite} frame Incoming frame to check for user relevance
 * @return {Boolean} Whether the frame is relevant for end developer debugging.
 */
function isRelevantFrame(frame) {
  var wrappedFrame = wrapCallSite(frame),
      generatedFile = frame.getFileName(),
      sourceFile = wrappedFrame.getFileName(),
      functionName = wrappedFrame.getFunctionName

  // Filter out all raw files which do not have any source mapping
  // This typically removes most of the build related infrastructure
  // which is neither application or framework code.
  ();
  if (!(sourceFile !== generatedFile)) {
    return false;
  }

  // Filter out specific functions e.g. webpack require wrappers
  if (FUNCTION_NAME_FILTERS.some(function (regexp) {
    return regexp.test(functionName);
  })) {
    return false;
  }

  // Filter out specific source files e.g. webpack bootstrap implementation
  if (SOURCE_FILE_FILTERS.some(function (regexp) {
    return regexp.test(sourceFile);
  })) {
    return false;
  }

  return true;
}

/**
 * Filters the existing list of CallSites to drop non user-relevant ones.
 * @param {CallSite[]} frames List of CallSite objects
 * @return {CallSite[]} Filtered CallSite list
 */
function getRelevantFrames(frames) {
  return frames.filter(isRelevantFrame);
}

/**
 * Processes a single CallSite entry to filter out build tool related or internal APIs.
 * During this process the method also uses source maps to figure out the original source location.
 * The CallSite API is documented in here: https://github.com/v8/v8/wiki/Stack-Trace-API#customizing-stack-traces
 * @param {CallSite} The current CallSite object to process.
 * @return {String} Stringified CallSite object with usage of source maps to refer source location
 */
function frameToString(frame) {
  var wrappedFrame = wrapCallSite(frame),
      sourceFile = wrappedFrame.getFileName(),
      functionName = wrappedFrame.getFunctionName(),
      typeName = wrappedFrame.getTypeName

  // Ignore some cryptic function names which typically are just function calls
  ();

  if (FUNCTION_NAME_CLEARANCE.some(function (regexp) {
    return regexp.test(functionName);
  })) {
    functionName = "";
  }

  // Filter out configured type names
  if (TYPE_NAME_CLEARANCE.some(function (regexp) {
    return regexp.test(typeName);
  })) {
    typeName = "";
  }

  // Strip out generated filename part from source field
  sourceFile = cleanSourceFileName(sourceFile

  // Retrieve source file locations
  );var lineNumber = wrappedFrame.getLineNumber(),
      columnNumber = wrappedFrame.getColumnNumber(),
      identifier = functionName || typeName || "<anonymous>";


  // Stack frames are displayed in the following format:
  //   at FunctionName (<Fully-qualified name/URL>:<line number>:<column number>)
  // Via: https://docs.microsoft.com/en-us/scripting/javascript/reference/stack-property-error-javascript
  return "at " + identifier + "@" + sourceFile + ":" + lineNumber + ":" + columnNumber;
}

/**
 * Build the stack property for V8 as V8 is documented to use whatever this call returns
 * as the value of the stack property.
 * @param {Error} nativeError Native JavaScript Error Object
 * @param {CallSite[]} structuredStackTrace a structured representation of the stack
 * @return {String} Generated `stack` property for error object
 */
function prepareStackTrace(nativeError, structuredStackTrace) {
  var frames = getRelevantFrames(structuredStackTrace),
      firstFrame = frames[0];


  // Need the first frame for highlighting affected source code, sometimes that's not available.
  if (null != firstFrame) {
    var wrappedFirstFrame = wrapCallSite(firstFrame),
        sourceFile = cleanSourceFileName(wrappedFirstFrame.getFileName()),
        sourceText = "";

    try {
      sourceText = readFileSync(sourceFile, "utf-8");
    } catch (error) {}
    // Ignore errors


    // Generate highlighted code frame and attach it to the native error object (for later usage)
    if (sourceText) {
      var result = codeFrame(sourceText, wrappedFirstFrame.getLineNumber(), wrappedFirstFrame.getColumnNumber(), CODE_FRAME_OPTIONS);
      nativeError.code = result;
    }
  }

  return frames.map(function (frame) {
    return frameToString(frame);
  }).filter(function (item) {
    return null != item;
  }).join("\n");
}

/**
 * Highlights the given stacktrace object using `chalk
 * @param {Stack} Stacktrace result string
 * @return {String} Highlighted stack trace for NodeJS
 */
function highlightStack(stack) {
  return stack.split("\n").map(function (line) {
    if (line.startsWith("at ")) {
      /* eslint-disable max-params */
      return line.replace(/(at )(.*?)(@)(.*?):([0-9]+)(:)([0-9]+)/, function (match, intro, id, symbol, filename, lineNo, separator, columnNo) {
        return "  - " + chalk.white(id) + " " + chalk.dim(filename) + " [" + chalk.yellow(lineNo) + ":" + chalk.yellow(columnNo) + "]";
      });
    }

    return chalk.yellow(line);
  }).join("\n");
}

/**
 * Logs the given error to the NodeJS console
 * @param {Error} nativeError Native JavaScript Error Object
 */
function logError(nativeError) {
  /* eslint-disable no-console */
  if (nativeError instanceof Error) {
    // Triggering generating formatted stacktrace
    String(nativeError.stack);

    var formattedMessage = chalk.red(nativeError.name + ": " + nativeError.message),
        formattedStack = highlightStack(nativeError.stack

    // Optionally display source code except
    );
    if (nativeError.code) {
      console.error(formattedMessage + "\n\n" + nativeError.code + "\n\n" + formattedStack);
    } else {
      console.error(formattedMessage + "\n\n" + formattedStack);
    }
  } else {
    console.error(nativeError);
  }
}

/**
 * Enable enhanced stack traces
 */
function enableEnhancedStackTraces() {
  var debug = 0 < arguments.length && arguments[0] !== undefined ? arguments[0] : false;

  // Override native Promise API with faster and more developerr friendly bluebird
  global.Promise = require("bluebird"

  /* eslint-disable no-use-extend-native/no-use-extend-native */
  );Promise.config({
    longStackTraces: debug,
    warnings: debug
  }

  // Catch unhandled Promise rejections and pass them over to our log handler
  );process.on("unhandledRejection", function (reason) {
    return logError(reason);
  }

  // Catch uncaught exceptions and pass them over to our log handler
  );process.on("uncaughtException", function (error) {
    return logError(error);
  }

  // Enable by hooking into V8 Stacktrace API integration
  // https://github.com/v8/v8/wiki/Stack-Trace-API
  );Error.prepareStackTrace = prepareStackTrace;

  console.log("Activated enhanced stack traces");
}

// This file is just for exporting infrastructure to applications built upon this.

export { createExpressServer, createUniversalMiddleware$$1 as createUniversalMiddleware, addFallbackHandler, renderPage, enableEnhancedStackTraces, logError, createReduxStore, createRootReducer, emptyReducer, emptyMiddleware, emptyEnhancer, ssrReducer, getRegion, getLanguage, getLocale, deepFetch, createApolloClient, RouterConnector, routerReducer, ensureIntlSupport, ensureReactIntlSupport, createLazyComponent };
//# sourceMappingURL=node.classic.esmodule.js.map
